#ifndef __MiscTableTypes_h
#define __MiscTableTypes_h
//=============================================================================
//
//		Copyright (C) 1995 by Paul S. McCarthy and Eric Sunshine.
//				Written by Paul S. McCarthy and Eric Sunshine.
//							All Rights Reserved.
//
//		This notice may not be removed from this source code.
//
//		This object is included in the MiscKit by permission from the authors
//		and its use is governed by the MiscKit license, found in the file
//		"License.rtf" in the MiscKit distribution.	Please refer to that file
//		for a list of all applicable permissions and restrictions.
//		
//=============================================================================
//-----------------------------------------------------------------------------
// <misckit/MiscTableTypes.h>
//
//		Common types used for the MiscTableScroll object.
//
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// $Id: MiscTableTypes.h,v 1.3 95/10/14 16:38:17 zarnuk Exp $
// $Log:		MiscTableTypes.h,v $
//	Revision 1.3  95/10/14	16:38:17  zarnuk
//	Added #defines for the max value of each enumeration.
//	Corrected documentation concerning -title method.
//	
//	Revision 1.2  95/10/08	16:12:39  zarnuk
//	Cell* --> id.
//	
//	Revision 1.1  95/09/27	12:21:21  zarnuk
//	Initial revision
//	
//-----------------------------------------------------------------------------

#ifdef __cplusplus
# define MISC_TS_EXTERN_BEGIN(X)		extern X {
# define MISC_TS_EXTERN_END				}
# define MISC_TS_CLASS_DEF(X)			class X
#else
# define MISC_TS_EXTERN_BEGIN(X)
# define MISC_TS_EXTERN_END
# define MISC_TS_CLASS_DEF(X)			typedef struct X X
#endif

MISC_TS_EXTERN_BEGIN( "Objective-C" )
#import <appkit/graphics.h>
MISC_TS_EXTERN_END


typedef int MiscPixels;
typedef int MiscCoord_V;		// Visual coordinate.
typedef int MiscCoord_P;		// Physical coordinate.

#define MISC_MIN_PIXELS_SIZE	((MiscPixels) 10)
#define MISC_MAX_PIXELS_SIZE	((MiscPixels) 0x7FFF0000)

typedef enum
	{
	MISC_COL_BORDER,
	MISC_ROW_BORDER
	} MiscBorderType;

#define MISC_MAX_BORDER MISC_ROW_BORDER


typedef enum
	{
	MISC_NO_TITLE,				// No titles on row/col cells.
	MISC_NUMBER_TITLE,			// Titles are sequential numbers.
	MISC_ALPHA_TITLE,			// Titles are sequential alphabetics...
	MISC_CUSTOM_TITLE,			// Titles are user-supplied strings...
	MISC_DELEGATE_TITLE			// Ask the delegate for titles.
	} MiscTableTitleMode;

#define MISC_MAX_TITLE	MISC_DELEGATE_TITLE


typedef enum
	{
	MISC_LIST_MODE,
	MISC_RADIO_MODE,
	MISC_HIGHLIGHT_MODE
	} MiscSelectionMode;

#define MISC_MAX_MODE	MISC_HIGHLIGHT_MODE


typedef enum
	{
	MISC_TABLE_CELL_TEXT,
	MISC_TABLE_CELL_ICON,
	MISC_TABLE_CELL_BUTTON,
	MISC_TABLE_CELL_CALLBACK
	} MiscTableCellStyle;

#define MISC_TABLE_CELL_MAX		MISC_TABLE_CELL_CALLBACK


#define MISC_SIZING_SPRINGY_BIT (1 << 0) // Adjusts for global limits.
#define MISC_SIZING_DATA_BIT	(1 << 1) // Expands to size of data.
#define MISC_SIZING_USER_BIT	(1 << 2) // User can resize.


typedef enum
	{
	MISC_NUSER_NDATA_NSPRINGY_SIZING,
	MISC_NUSER_NDATA_SPRINGY_SIZING,
	MISC_NUSER_DATA_NSPRINGY_SIZING,
	MISC_NUSER_DATA_SPRINGY_SIZING,
	MISC_USER_NDATA_NSPRINGY_SIZING,
	MISC_USER_NDATA_SPRINGY_SIZING,
	MISC_USER_DATA_NSPRINGY_SIZING,
	MISC_USER_DATA_SPRINGY_SIZING,
	} MiscTableSizing;

#define MISC_MAX_SIZING MISC_USER_DATA_SPRINGY_SIZING


typedef enum
	{
	MISC_SORT_ASCENDING,
	MISC_SORT_DESCENDING
	} MiscSortDirection;

#define MISC_SORT_DIR_MAX		MISC_SORT_DESCENDING

typedef enum							// Default selector used to get
	{									//		data from cell:
	MISC_SORT_STRING_CASE_INSENSITIVE,	// -title, then -stringValue
	MISC_SORT_STRING_CASE_SENSITIVE,	// -title, then -stringValue
	MISC_SORT_INT,						// -intValue
	MISC_SORT_UNSIGNED_INT,				// -intValue
	MISC_SORT_TAG,						// -tag
	MISC_SORT_UNSIGNED_TAG,				// -tag
	MISC_SORT_FLOAT,					// -floatValue
	MISC_SORT_DOUBLE,					// -doubleValue
	MISC_SORT_SKIP,						// Don't compare cells in this slot.
	} MiscSortType;

#define MISC_SORT_TYPE_MAX		MISC_SORT_SKIP

@class MiscTableScroll;
@class Cell;

//-----------------------------------------------------------------------------
// MiscCompareCellFunc
//
//		Compare two cells, given pointers to the cells.
//
// Returns:
//		< 0		if (x < y)
//		= 0		if (x = y)
//		> 0		if (x > y)
//
//		There is a default for each of the builtin types of comparisons.  
//
//		MiscDefaultCompareCellFunc[ MISC_SORT_STRING_IGNORE_CASE ]
//				Will compare two cells by using a non-case-sensitive string 
//				comparison.	 If the cell respondsTo: '-title' that is used,
//				otherwise '-stringValue' is used to get the string values
//				from the cells.
//
//		MiscDefaultCompareCellFunc[ MISC_SORT_STRING_CASE_SENSITIVE ]
//				Will compare two cells by using a case-sensitive string
//				comparison.	 If the cell respondsTo: '-title' that is used,
//				otherwise '-stringValue' is used to get the string values
//				from the cells.	 
//
//		MiscDefaultCompareCellFunc[ MISC_SORT_INT ]
//				Will compare two cells by comparing the values returned
//				from the '-intValue' message, interpreting them as signed
//				integer values.
//
//		MiscDefaultCompareCellFunc[ MISC_SORT_UNSIGNED_INT ]
//				Will compare two cells by comparing the values returned
//				from the '-intValue' message, interpreting them as unsigned
//				integer values.
//
//		MiscDefaultCompareCellFunc[ MISC_SORT_TAG ]
//				Will compare two cells by comparing the values returned
//				from the '-tag' message, interpreting them as signed
//				integer values.
//
//		MiscDefaultCompareCellFunc[ MISC_SORT_UNSIGNED_TAG ]
//				Will compare two cells by comparing the values returned
//				from the '-tag' message, interpreting them as unsigned
//				integer values.
//
//		MiscDefaultCompareCellFunc[ MISC_SORT_FLOAT ]
//				Will compare two cells by comparing the values returned
//				from the '-floatValue' message.
//
//		MiscDefaultCompareCellFunc[ MISC_SORT_DOUBLE ]
//				Will compare two cells by comparing the values returned
//				from the '-doubleValue' message.
//
//		MiscDefaultCompareCellFunc[ MISC_SORT_SKIP ]
//				Will not compare the two cells.	 Always returns 0 (equal).
//
//-----------------------------------------------------------------------------
typedef int (*MiscCompareCellFunc)( MiscTableScroll* table, id x, id y );

extern MiscCompareCellFunc MiscDefaultCompareCellFunc[ MISC_SORT_SKIP + 1 ];


//-----------------------------------------------------------------------------
// MiscCompareEntryFunc
//
//		Compare two cells, given the coordinates of the two cells.	The two 
//		entries must be from either the same row, or the same column.  
//
// Returns:
//		< 0		if (table[row1][col1] < table[row2][col2])
//		= 0		if (table[row1][col1] = table[row2][col2])
//		> 0		if (table[row1][col1] > table[row2][col2])
//
//		The default version, MiscDefaultCompareEntryFunc(), will invoke the 
//		correct MiscDefaultCompareCellFunc[] function based on the sort type 
//		that is set for the slot that the two entries share.  If there is a 
//		user-installed custom sort function for the slot, the custom sort 
//		function will be used instead of the default function.	Custom 
//		compare functions are called with pointers to the two cells.  
//
//-----------------------------------------------------------------------------
typedef int (*MiscCompareEntryFunc)
		( MiscTableScroll* table, int row1, int col1, int row2, int col2 );

extern MiscCompareEntryFunc MiscDefaultCompareEntryFunc;


//-----------------------------------------------------------------------------
// MiscCompareSlotFunc
//
//		Compare two entire slots (usually rows) from the table.
//
// Returns:
//		< 0		if (table.b[slot1] < table.b[slot2])
//		= 0		if (table.b[slot1] = table.b[slot2])
//		> 0		if (table.b[slot1] > table.b[slot2])
//
//		The default version, MiscDefaultComareSlotFunc(), will compare the
//		slots, one component at a time using whatever compare entry function
//		is set in the table.
//
//		The components are visited in the current sort-order array order.
//		If no sort-order array has been set for the table, the components
//		are visited in the current visual order.
//
//-----------------------------------------------------------------------------
typedef int (*MiscCompareSlotFunc)
		( MiscTableScroll* table, MiscBorderType b, int slot1, int slot2 );


extern MiscCompareSlotFunc	MiscDefaultCompareSlotFunc;

#endif // __MiscTableTypes_h
